@@ -171,8 +171,13 @@
 #define TB_CONFIG_POSIX_HAVE_FDATASYNC 1
 /* #undef TB_CONFIG_POSIX_HAVE_COPYFILE */
 #define TB_CONFIG_POSIX_HAVE_SENDFILE 1
-#define TB_CONFIG_POSIX_HAVE_EPOLL_CREATE 1
-#define TB_CONFIG_POSIX_HAVE_EPOLL_WAIT 1
+#ifdef __COSMOPOLITAN__
+/* #undef TB_CONFIG_POSIX_HAVE_EPOLL_CREATE */
+/* #undef TB_CONFIG_POSIX_HAVE_EPOLL_WAIT */
+#else
+#   define TB_CONFIG_POSIX_HAVE_EPOLL_CREATE 1
+#   define TB_CONFIG_POSIX_HAVE_EPOLL_WAIT 1
+#endif
 #if defined(__ANDROID__)
 /* #undef TB_CONFIG_POSIX_HAVE_POSIX_SPAWNP */
 #else
@@ -196,7 +201,11 @@
 #define TB_CONFIG_POSIX_HAVE_FCNTL 1
 #define TB_CONFIG_POSIX_HAVE_PIPE 1
 /* #undef TB_CONFIG_POSIX_HAVE_PIPE2 */
-#define TB_CONFIG_POSIX_HAVE_MKFIFO 1
+#ifdef __COSMOPOLITAN__
+/* #undef TB_CONFIG_POSIX_HAVE_MKFIFO */
+#else
+#   define TB_CONFIG_POSIX_HAVE_MKFIFO 1
+#endif
 #define TB_CONFIG_POSIX_HAVE_FUTIMENS 1
 #define TB_CONFIG_POSIX_HAVE_UTIMENSAT 1
 
@@ -211,6 +220,10 @@
 /* #undef TB_CONFIG_SYSTEMV_HAVE_VALGRIND_STACK_REGISTER */
 
 // linux functions
-#define TB_CONFIG_LINUX_HAVE_INOTIFY_INIT 1
+#ifdef __COSMOPOLITAN__
+/* #undef TB_CONFIG_LINUX_HAVE_INOTIFY_INIT */
+#else
+#   define TB_CONFIG_LINUX_HAVE_INOTIFY_INIT 1
+#endif
 
 #endif
