@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        download.lua
@@ -34,9 +34,21 @@ import("net.proxy")
 import("devel.git")
 import("utils.archive")
 
+-- get url extension
+function _url_extension(url)
+    local extension = archive.extension(url)
+    if extension == "" then
+        -- maybe non-archive file, e.g. .exe, .sh, ..
+        local urlpath = url:split('?', {plain = true})[1]
+        extension = path.extension(urlpath)
+    end
+    return extension
+end
+
 -- checkout codes from git
 function _checkout(package, url, sourcedir, opt)
     opt = opt or {}
+    local filename = opt.url_filename or url_filename(url)
 
     -- we need to enable longpaths on windows
     local longpaths = package:policy("platform.longpaths")
@@ -62,7 +74,7 @@ function _checkout(package, url, sourcedir, opt)
     -- we can use local package from the search directories directly if network is too slow
     local localdir
     local searchnames = {package:name() .. archive.extension(url),
-                         path.basename(url_filename(url))}
+                         path.basename(filename)}
     for _, searchname in ipairs(searchnames) do
         localdir = find_directory(searchname, core_package.searchdirs())
         if localdir then
@@ -148,9 +160,12 @@ function _download(package, url, sourcedir, opt)
     opt = opt or {}
 
     -- get package file
-    local packagefile = url_filename(url)
-    if opt.download_only then
-        packagefile = package:name() .. "-" .. package:version_str() .. archive.extension(packagefile)
+    local packagefile = opt.url_filename
+    if not packagefile then
+        packagefile = url_filename(url)
+        if not os.isfile(packagefile) then -- we need to be compatible with the old file names
+            packagefile = package:name() .. "-" .. package:version_str() .. _url_extension(packagefile)
+        end
     end
 
     -- get sourcehash from the given url
@@ -177,7 +192,7 @@ function _download(package, url, sourcedir, opt)
             os.cp(url, packagefile)
         else
             local localfile
-            local searchnames = {package:name() .. "-" .. package:version_str() .. archive.extension(url),
+            local searchnames = {package:name() .. "-" .. package:version_str() .. _url_extension(url),
                                  packagefile}
 
             -- match github name mangling https://github.com/xmake-io/xmake/issues/1343
@@ -213,6 +228,12 @@ function _download(package, url, sourcedir, opt)
 
     -- we do not extract it if we download only it.
     if opt.download_only then
+        tty.erase_line_to_start().cr()
+        if cached then
+            cprint("${yellow}  => ${clear}download %s .. ${color.success}cached", url)
+        else
+            cprint("${yellow}  => ${clear}download %s .. ${color.success}${text.success}", url)
+        end
         return
     end
 
@@ -222,9 +243,9 @@ function _download(package, url, sourcedir, opt)
     local extension = archive.extension(packagefile)
     local errors
     local ok = try {
-        function() 
+        function()
             archive.extract(packagefile, sourcedir_tmp, {excludes = opt.url_excludes})
-            return true 
+            return true
         end,
         catch {
             function (errs)
@@ -257,6 +278,7 @@ function _download(package, url, sourcedir, opt)
         raise(errors or string.format("cannot extract %s, maybe missing extractor or invalid package file!", packagefile))
     else
         -- if it is not archive file, we only need to create empty source directory and use package:originfile()
+        -- e.g. .exe, .sh
         os.tryrm(sourcedir)
         os.mkdir(sourcedir)
     end
@@ -326,6 +348,7 @@ function main(package, opt)
         local url_includes = package:url_includes(url)
         local url_http_headers = package:url_http_headers(url)
         local url_submodules = package:extraconf("urls", url, "submodules")
+        local url_filename_custom = package:extraconf("urls", url, "filename")
 
         -- filter url
         url = filter.handle(url, package)
@@ -360,19 +383,22 @@ function main(package, opt)
                         url_alias = url_alias,
                         url_excludes = url_excludes,
                         url_includes = url_includes,
-                        url_submodules = url_submodules})
+                        url_submodules = url_submodules,
+                        url_filename = url_filename_custom})
                 elseif git.checkurl(url) then
                     _checkout(package, url, sourcedir, {
                         url_alias = url_alias,
                         url_includes = url_includes,
-                        url_submodules = url_submodules})
+                        url_submodules = url_submodules,
+                        url_filename = url_filename_custom})
                 else
                     _download(package, url, sourcedir, {
                         download_only = opt.download_only,
                         url_alias = url_alias,
                         url_excludes = url_excludes,
                         url_includes = url_includes,
-                    url_http_headers = url_http_headers})
+                        url_http_headers = url_http_headers,
+                        url_filename = url_filename_custom})
                 end
                 return true
             end,
@@ -410,7 +436,7 @@ function main(package, opt)
                                     searchnames:insert(package:name() .. archive.extension(url_failed))
                                     searchnames:insert(path.basename(url_filename(url_failed)))
                                 else
-                                    local extension = archive.extension(url_failed)
+                                    local extension = _url_extension(url_failed)
                                     if extension then
                                         searchnames:insert(package:name() .. "-" .. package:version_str() .. extension)
                                     end
@@ -432,15 +458,6 @@ function main(package, opt)
         }
 
         if ok then
-            -- download only packages, we need not to install it, so we need flush console output
-            if opt.download_only then
-                tty.erase_line_to_start().cr()
-                if git.checkurl(url) then
-                    cprint("${yellow}  => ${clear}clone %s %s .. ${color.success}${text.success}", url, package:version_str())
-                else
-                    cprint("${yellow}  => ${clear}download %s .. ${color.success}${text.success}", url)
-                end
-            end
             break
         end
     end
